@@ -2,6 +2,7 @@ use std::env;
 use std::path::Path;
 
 use anyhow::Result;
+use cloudflare::endpoints::workerskv::write_bulk::KeyValuePair;
 use indicatif::{ProgressBar, ProgressStyle};
 use reqwest::blocking::Client;
 use serde::{Deserialize, Serialize};
@@ -10,6 +11,7 @@ use crate::build::build_target;
 use crate::deploy::{self, DeploymentSet};
 use crate::http::{self, Feature};
 use crate::kv::bulk;
+use crate::kv::key::get_value;
 use crate::settings::global_user::GlobalUser;
 use crate::settings::toml::migrations::{MigrationTag, Migrations};
 use crate::settings::toml::Target;
@@ -73,7 +75,8 @@ pub fn publish(
 
         let site_namespace = sites::add_namespace(user, target, false)?;
 
-        let (to_upload, asset_manifest) = sites::sync(target, user, &site_namespace.id, path)?;
+        let (to_upload, to_delete, asset_manifest) =
+            sites::sync(target, user, &site_namespace.id, path)?;
 
         // First, upload all existing files in bucket directory
         StdErr::working("Uploading site files");
@@ -104,6 +107,57 @@ pub fn publish(
         upload::script(&upload_client, target, Some(asset_manifest))?;
 
         run_deploy(target)?;
+
+        // Finally, mark any stale files for expiration
+        if !to_delete.is_empty() {
+            StdErr::info("Deleting stale files...");
+
+            let delete_progress_bar = if to_delete.len() > bulk::BATCH_KEY_MAX {
+                let delete_progress_bar = ProgressBar::new(to_delete.len() as u64);
+                delete_progress_bar.set_style(
+                    ProgressStyle::default_bar().template("{wide_bar} {pos}/{len}\n{msg}"),
+                );
+                Some(delete_progress_bar)
+            } else {
+                None
+            };
+
+            let account_id = target.account_id.load()?;
+
+            // create identical key:value pairs, except set them to expire in five minutes
+            // isn't this very slow? yes. can we do this in bulk? unfortunately as of now
+            // there isn't a KV endpoint for "bulk read".
+            // https://api.cloudflare.com/#workers-kv-namespace-properties
+            let five_minutes_from_now = chrono::Utc::now().timestamp() + (60 * 5);
+            let http_client = reqwest::blocking::Client::new();
+            let to_delete_pairs = to_delete
+                .into_iter()
+                .map(|key| {
+                    get_value(&key, &site_namespace.id, account_id, user, &http_client).map(
+                        // discard the timestamp to match wrangler2 implementation
+                        |(value, _)| KeyValuePair {
+                            key,
+                            value,
+                            expiration: Some(five_minutes_from_now),
+                            expiration_ttl: None,
+                            base64: None,
+                        },
+                    )
+                })
+                .collect::<Result<Vec<_>>>()?;
+
+            bulk::put(
+                target,
+                user,
+                &site_namespace.id,
+                to_delete_pairs,
+                &delete_progress_bar,
+            )?;
+
+            if let Some(pb) = delete_progress_bar {
+                pb.finish_with_message("Done deleting");
+            }
+        }
     } else {
         let upload_client = http::legacy_auth_client(user);
 
